#ifndef LIB_LANDSCAPE_H
#define LIB_LANDSCAPE_H

#include "math/vec.h"
#include "base/texture.h"
#include "lib/collision.h"

class Landscape {
protected:

	typedef struct {
		//calculated, depending on the height
		vec3f	color;

		//generated via fractal
		float	height;
	} vtx_t;

	//landscape vertex data:
	vtx_t *vtx;

	int detail;

	int list_id;

	float maxvalue;
	float minvalue;

	void reset();

	void renderImmediateData();

public:

	//components:

	Texture tex;
	float texDetailX;
	float texDetailY;

	vec3f translate;
	vec3f scale;

	float restitution;
	float friction;

	//startup/shutdown

	Landscape() { reset(); }
	~Landscape() { shutdown(); }

	bool init(int detail, int filter_level, int seed = 0xdeadbeef);
	void shutdown();

	//mod data

	void updateMesh();

	//v is in world coordinates.  Landscape:: will have to inverse-transform them by its basis
	void gaussFalloff(const vec3f &v);

	//rendering

	void render();
	void compile();

	//collision

	bool testSphereCollision(const vec3f &src, float rad, col_t *col);
};

#endif
