#ifndef LIB_OBJINST_H
#define LIB_OBJINST_H

#include "math/vec.h"
#include "base/texture.h"
#include "lib/objfile.h"
#include "lib/collision.h"

/**
 * our object instance's material settings:
 */
class Material {
public:
	Material() {}
	Material(const vec4f &amb, const vec4f &diff, const vec4f &spec, float shine) {
		set(amb,diff,spec,shine);
	}

	void set(const vec4f &amb, const vec4f &diff, const vec4f &spec, float shine) {
		this->amb = amb;
		this->diff = diff;
		this->spec = spec;
		this->shine = shine;	
	}	

	vec4f	amb;
	vec4f	diff;
	vec4f	spec;

	/**
	 * "love is in the water,
	 *  love is in the air.
	 *  teach me where to go,
	 *  tell me will love be there..."
	 *		--Collective Soul, "Shine".
	 */
	float	shine;

	//todo - vec3f emm

	void setup(int face);
};

/**
 * our object instance:
 */
class ObjInst {

protected:

	typedef struct bspnode_s {
		float	mid;	//value of dividing plane for children
		char	axis;	//0,1,2 <=> x,y,z , -1 = leaf
		int		tri;	//index of the first child for this node.  each's "next" field points to the next in this node
	} bspnode_t;

	typedef struct {
		bool	vis;		//whether this tri is visible or not
		int		next;		//index of the next triangle within this BSP node
	} triextra_t;

	//instance model
	ObjFile *model;

	//array of which faces are visible
	triextra_t *triextra;
//	unsigned char *vis;
	vec3f		*vtxextra;	//buffer the transformed object

	bspnode_t	*bspnode;
	int			bspnodeCount;

	bool testTreeSphereCollision(const vec3f &src, float rad, col_t *col, int node);

	/**
	 * number of bytes in vis
	 * sized model.tri.size()
	 */
	int		visCount;

	void	buildBSPNode(int node, box3f box);
	void	addTriToBSP(int i, const box3f &box, int node = 0);

	void	reset();

	char	title[256];

	box3f	bbox;

public:

	//cubemap texture
	Texture cubeTex;

	//2D texture
	Texture tex;

	//replacement for color/specular soon...
	Material mat;

	//reflect coefficient
	float reflect;

	//orthonormal basis w/transform
	basis34f basis;

	//non-uniform scale components, relative to current orientation
	vec3f scale;

	//collision
	float restitution;
	float friction;

	ObjInst() { reset(); }
	~ObjInst() { unload(); }

	//required for updateBBox and buildBSPTree
	//transforms all vertices and stores them here:
	void calcVtxs();

	void updateBBox();
	box3f &getBBox() { return bbox; }

	//axis aligned BSP tree
	bool buildBSPTree(int nodeCount);

	bool testSphereCollision(const vec3f &src, float rad, col_t *col);

	/**
	 * unload all data
	 */
	void unload();

	/**
	 * render
	 */
	void render();

	/**
	 * useful method for shadow casting.
	 */
	void renderEdges(const vec3f &offset, float pointSource, float dist) const;
	void renderEdges(const vec4f &offset, float dist) const {
		renderEdges((vec3f&)offset, offset.w, dist);
	}

	/**
	 * calculates the bitflags of which faces are visible
	 * from the source position.  uses front/back testing for each tri's plane
	 */
	void calcVisibleFaces(const vec4f &src);

	/**
	 * converts the vec3f from world space to this object's space
	 * same as inverse transforming the OpenGL space
	 *
	 * TODO - move this to the basis class
	 */
	vec3f calcRelative(const vec3f &src);
	vec4f calcRelative(const vec4f &src);	//src->w == 0 implies vector, w==1 implies point source

	//interface
	ObjFile *getModel() const { return model; }
	bool setModel(ObjFile *model);

	const char *getTitle() { return title; }
};

#endif
