#ifndef LIB_PARSYSTEM_H
#define LIB_PARSYSTEM_H

#include "math/vec.h"

class Particle {
protected:
	//accumulated force
	vec3f	f;

	//sum of damage on this particle:
	double	dmg;

public:
	//system: bidirectional link
	Particle *prev, *next;

	//physics
	float	massInv;	//inverse of the mass
	vec3f	x;			//position
	vec3f	v;			//velocity

	//previous state
	vec3f	srcX, srcV;

	//RK4 dummy vars
	vec3f	tmpV[4], tmpF[4];

	//interface:

	void addForce(const vec3f &force) {
		f += force;
	}

	const vec3f &getForce() const { return f;}

	void clearForce() {
		f.set(0,0,0);
	}

	double getDamage() { return dmg; }
	void clearDamage() { dmg = 0; }
	void addDamage(double d) { dmg += d; }
};

class ParSystem {
protected:
	void reset();

	int parCount;
	int liveCount;
	Particle *live, *dead;
public:
	ParSystem() {reset();}
	~ParSystem() {shutdown();}

	bool init();
	void shutdown();

	Particle *createParticle();
	void removeParticle(Particle *p);

	Particle *getLive() { return live; }
	int getLiveCount() { return liveCount; }
	int getTotalCount() { return parCount; }
};


#endif
