#ifndef LIB_SPRINGOBJ_H
#define LIB_SPRINGOBJ_H

#include <vector>
#include "lib/parsystem.h"
#include "lib/springsystem.h"

class SpringObj {
protected:
	std::vector<Particle *> vertex;
	std::vector<int> spring;

	ParSystem *parSys;
	SpringSystem *springSys;

	//clear() should always null all pointers and return all data to its default
	virtual void clear();

public:

	//the constructor should always contain a call to clear() to null all pointers and return all data to its default
	SpringObj() {clear();}

	//the destructor should always call shutdown() to unalloc and remove any existing data
	~SpringObj() {shutdown();}

	//init() should always begin with shutdown() to unalloc and remove any existing data
	virtual bool init(ParSystem *parSys, SpringSystem *springSys);

	//shutdown() should always end with clear() to null all pointers and return all data to its default
	virtual void shutdown();

	//space transforms.  relative to identity (left-multiply-transpose-vector-style)
	void rotate(const quatf &q);	//quaternion =)
	void translate(const vec3f &t);
	void scale(const vec3f &s);
	virtual void resetSpringLengths();

	void setSpringKs(float ks);
	void setSpringKd(float kd);

	//vertex accexx
	int getVertexCount() { return (int)vertex.size(); }
	Particle *getVertex(int i) { return vertex[i];}

};

#endif
